@@ -8,11 +8,13 @@ import re
 from django.conf.urls import url
 from django.contrib.admin import ModelAdmin
 from django.contrib.admin.options import csrf_protect_m
+from django.contrib.admin.templatetags.admin_urls import add_preserved_filters
 from django.contrib.admin.utils import unquote
 from django.http import response
 from django.template.response import SimpleTemplateResponse
-from django.urls import resolve
+from django.urls import NoReverseMatch, resolve, reverse
 from django.utils.text import capfirst
+from six.moves.urllib.parse import parse_qsl, urlparse
 
 from .sites import CustomObjectToolAdminSiteMixin
 from .utils import object_tool_context
@@ -35,6 +37,7 @@ class CustomObjectToolModelAdminMixin(object):
 
     @property
     def _base_change_list_template(self):
+        """parent change list template"""
         parent = super(CustomObjectToolModelAdminMixin, self)
         return getattr(parent, "change_list_template", None)\
             or "admin/change_list.html"
@@ -43,6 +46,7 @@ class CustomObjectToolModelAdminMixin(object):
 
     @property
     def _base_change_form_template(self):
+        """parent change form template"""
         parent = super(CustomObjectToolModelAdminMixin, self)
         return getattr(parent, "change_form_template", None)\
             or "admin/change_form.html"
@@ -61,29 +65,58 @@ class CustomObjectToolModelAdminMixin(object):
         info = self.model._meta.app_label, self.model._meta.model_name
         urlpatterns = [
             url(
-                r"^objecttool/(.+)?$",
+                r"^(?:(?P<object_id>.+)/)?objecttool/(?P<action_name>.+)/$",
                 wrap(self.object_tool_view),
                 name="%s_%s_objecttool" % info
             )
         ] + urlpatterns
         return urlpatterns
 
-    def _get_request_view(self, request):
-        """util for get request view"""
+    def _get_view_name(self, request):
+        """get request url name, returns 'changelist' or 'change'"""
+        pattern = "_([^_]+?)$"
         url_name = resolve(request.path_info).url_name
-        match = re.search("_([^_]+?)$", url_name)
+        match = re.search(pattern, url_name)
         view = match.group(1)
         if view == "objecttool":
-            view = request.POST["object-tool-referrer-view"]
+            referrer = self._get_post_objecttool_url(request)
+            url_name = resolve(urlparse(referrer).path).url_name
+            match = re.search(pattern, url_name)
+            view = match.group(1)
         return view
 
+    def _get_post_objecttool_url(self, request):
+        """get redirect url after complete objecttool action"""
+        rv = request.META.get("HTTP_REFERRER")
+        if not rv:
+            opts = self.model._meta
+            obj_id = resolve(request.path).kwargs.get("object_id")
+            if obj_id:
+                url_name = "admin:%s_%s_change" % (
+                    opts.app_label, opts.model_name)
+                try:
+                    post_url = reverse(
+                        url_name, current_app=self.admin_site.name,
+                        kwargs=dict(object_id=obj_id))
+                except NoReverseMatch:
+                    post_url = reverse(
+                        url_name, current_app=self.admin_site.name,
+                        args=(obj_id,))
+            else:
+                post_url = reverse("admin:%s_%s_changelist" %
+                    (opts.app_label, opts.model_name),
+                    current_app=self.admin_site.name)
+            preserved_filters = self.get_preserved_filters(request)
+            rv = add_preserved_filters(
+                {'preserved_filters': preserved_filters, 'opts': opts}, post_url
+            )
+        return rv
+
     def _get_base_object_tools(self, request):
-        """
-        Return the list of object tools
-        """
+        """return the list of object tools"""
         object_tools = []
 
-        view = self._get_request_view(request)
+        view = self._get_view_name(request)
 
         # Gather object tools from the admin site first
         if isinstance(self.admin_site, CustomObjectToolAdminSiteMixin):
@@ -166,47 +199,40 @@ class CustomObjectToolModelAdminMixin(object):
             description = capfirst(object_tool.replace("_", " "))
         return func, object_tool, description
 
-    def response_object_tool(self, request, obj=None, extra_context=None):
-        """
-        Handle an admin object tool.
-        """
-        object_tools = self.get_object_tools(request)
-        object_tool = object_tools[request.POST["object-tool"]]
-        func = object_tool[0]
-        rv = func(self, request, obj)
+    def response_object_tool(self, request, action, obj=None, extra_context=None):
+        """Handle an admin object tool"""
+        rv = action(self, request, obj)
         if isinstance(rv, SimpleTemplateResponse):
             rv.context_data = rv.context_data or dict()
-            rv.context_data.update(extra_context)
+            extra_context and rv.context_data.update(extra_context)
             return rv
-        if isinstance(rv, response.HttpResponseBase):
+        elif isinstance(rv, response.HttpResponseBase):
             return rv
         else:
-            redirect_url = request.POST["object-tool-referrer-url"]
+            redirect_url = self._get_post_objecttool_url(request)
             return response.HttpResponseRedirect(redirect_url)
 
     @csrf_protect_m
-    def object_tool_view(self, request, object_id=None, extra_context=None):
-        # handle object tool behaviors
-        if "object-tool" not in request.POST:
-            return response.HttpResponseBadRequest()
-
-        name = request.POST["object-tool"]
-        if name not in self.get_object_tools(request):
+    def object_tool_view(self, request, action_name, object_id=None, extra_context=None):
+        if action_name not in self.get_object_tools(request):
             return response.HttpResponseForbidden()
 
-        object_tool = self.get_object_tool(name)
-        allow_get = not getattr(object_tool, "allow_get", False)
+        action = self.get_object_tool(action_name)[0]
+        allow_get = getattr(action, "allow_get", False)
         if not allow_get and request.method != "POST":
             return response.HttpResponseNotAllowed()
 
         obj = object_id and self.get_object(request, unquote(object_id))
-        return self.response_object_tool(request, obj, extra_context)
+        return self.response_object_tool(request, action, obj, extra_context)
 
     @csrf_protect_m
     def changelist_view(self, request, extra_context=None):
         extra_context = self._prepare_object_tool_view(request, extra_context)
         extra_context.update(
-            object_tool_base_template=self._base_change_list_template
+            object_tool_base_template=self._base_change_list_template,
+            changelist_filters=dict(
+                parse_qsl(self.get_preserved_filters(request))
+            ).get("_changelist_filters", "")
         )
         return super(CustomObjectToolModelAdminMixin, self).changelist_view(
             request, extra_context)
@@ -215,7 +241,10 @@ class CustomObjectToolModelAdminMixin(object):
     def changeform_view(self, request, object_id=None, form_url="", extra_context=None):
         extra_context = self._prepare_object_tool_view(request, extra_context)
         extra_context.update(
-            object_tool_base_template=self._base_change_form_template
+            object_tool_base_template=self._base_change_form_template,
+            changelist_filters=dict(
+                parse_qsl(self.get_preserved_filters(request))
+            ).get("_changelist_filters", "")
         )
         return super(CustomObjectToolModelAdminMixin, self).changeform_view(
             request, object_id, form_url, extra_context)
@@ -226,9 +255,7 @@ class CustomObjectToolModelAdminMixin(object):
         extra_context.update(
             object_tools=tuple(map(
                 lambda o: object_tool_context(*o),
-                self.get_object_tools(request).values())),
-            object_tool_referrer_url=request.get_full_path(),
-            object_tool_referrer_view=self._get_request_view(request)
+                self.get_object_tools(request).values()))
         )
         return extra_context
 
