@@ -1,9 +1,9 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
 
-
-OBJECTTOOL_ALLOWED_PROPERTIES = (
-    "__name__", "classes", "help_text", "short_description")
+OBJECTTOOL_LINK_ALLOWED_PROPERTIES = ("href", "target")
+OBJECTTOOL_ALLOWED_PROPERTIES = OBJECTTOOL_LINK_ALLOWED_PROPERTIES + (
+    "__name__", "allow_get", "classes", "help_text", "short_description")
 
 
 def object_tool_context(func, name, short_description):
